package miage.ter.projet_a_remplacer.model;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpSessionBindingEvent;
import javax.servlet.http.HttpSessionBindingListener;


/**
 * Cette classe représente une connexion à la base de données. Elle encapsule
 * les méthodes permettant de lire et écrire les données Elle communique avec le
 * monde extérieur en utilisant des objets Java standard (int, String, …) ou des
 * classes du modèle (Point).
 * 
 * Les objets de cette classe seront stockés dans la session HTTP. Chaque client
 * du serveur Web aura donc une connection à la base de donnée le temps de sa
 * session.
 * 
 * La classe est Serializable, elle peut être sauvegardée sur le disque si le
 * serveur Web veut mettre la session en cache. Évidemment la connection doit
 * alors être.
 * 
 * La classe est AutoClosable, sa méthode close() est appelée lorsque java n'a
 * plus besoin de l'objet, permettant de fermer la connexion à la BD proprement.
 * 
 * La classe implémente HttpSessionBindingListener pour pouvoir fermer la
 * connexion lorsque les objets sont retirés de la session HTTP ou que celle ci
 * est détruite.
 * 
 * Cette classe doit être renommée pour avoir un nom plus parlant (MagasinDB,
 * StoreDB, …)
 * 
 * Les premières méthodes privées doivent rester telle qu'elle, les méthodes
 * faisant des SELECT/UPDATE/INSERT doivent évidemment être adaptées.
 */

public class ExampleDB implements Serializable, AutoCloseable, HttpSessionBindingListener {

    // Si l'objet est sauvé sur le disque, cette attribut ne sera pas sauvé et mis à
    // null.
    private transient Connection cnx;
    private String host;
    private String port;
    private String name;
    private String user;
    private String pass;

    // Appelée lorsque l'objet va être mis sur le disque
    private void writeObject(ObjectOutputStream oos) throws IOException {
        close();
        oos.defaultWriteObject(); // Calling the default serialization logic
    }

    /** Crée une connexion si elle n'est pas déjà ouverte */

    private Connection getConnection() throws SQLException {
        if (cnx == null || cnx.isClosed()) {
            cnx = DriverManager.getConnection("jdbc:postgresql://" + host + ":" + port + "/" + name, user, pass);
        }
        return cnx;
    }

    /** Appelée lorsqu'on retire cet objet de la session. */
    @Override
    public void valueUnbound(HttpSessionBindingEvent event) {
        close();
    }

    /** Permet de fermer explicitement la connexion */
    public void close() {
        if (cnx != null) {
            try {
                cnx.close();
            } catch (SQLException e) {
                // déjà fermé ?
            }
            cnx = null;
        }
    }

    ///// PARTIE À MODIFIER ///////

    public ExampleDB(String host, String port, String name, String user, String pass) {
        // Renommer le constructeur et
        // initialiser les autres champs si besoin.
        this.host = host;
        this.port = port;
        this.name = name;
        this.user = user;
        this.pass = pass;
    }

    /**
     * Ajoute un point à la BD.
     * 
     * Le motif try (init) { … } catch ( …) { … } effectue les initialisations se
     * trouvant dans init, puis le code dans le bloc. En fin de bloc et même si des
     * exceptions sont levées, les variables initialisées dans init sont fermées en
     * appeleant leur méthode .close() Cette construction est utile avec des
     * fichiers, des connexions, …
     * 
     */
    public void addPoint(int x, int y) throws SQLException {
        try (Statement stmt = getConnection().createStatement()) {
            stmt.executeUpdate("INSERT INTO point VALUES(" + x + ", " + y + ")");
        }
    }

    /**
     * Une méthode qui retire les doublons. On illustre comment faire une
     * transaction.
     * 
     */
    public void deleteDuplicates() throws SQLException {

        try (Statement stmt = getConnection().createStatement()) {

            getConnection().setAutoCommit(false); // BEGIN

            stmt.executeUpdate("SELECT DISTINCT * INTO tmp_table FROM point");
            stmt.executeUpdate("DELETE FROM point"); // Pas de WHERE, on efface tout
            stmt.executeUpdate("INSERT INTO point (SELECT * FROM tmp_table)");
            stmt.executeUpdate("DROP TABLE tmp_table");

            getConnection().commit(); // COMMIT
        } catch (SQLException e) {
            getConnection().rollback(); // ROLLBACK
            throw e; // On relance e pour quitter la méthode en erreur
        }

    }

    /**
     * Une méthode qui renvoie tous les points.
     */

    public List<Point> getAllPoints() throws SQLException {
        List<Point> lst = new ArrayList<>();
        try (Statement stmt = getConnection().createStatement()) {
            ResultSet res = stmt.executeQuery("SELECT * FROM point");
            while (res.next()) {
                lst.add(new Point(res.getInt("x"), res.getInt("y")));
            }
        }
        return lst;
    }

}
